/****************************************************************************
** Meta object code from reading C++ file 'board.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/tetzle-2.1.1/src/board.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'board.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_Board_t {
    QByteArrayData data[30];
    char stringdata0[301];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Board_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Board_t qt_meta_stringdata_Board = {
    {
QT_MOC_LITERAL(0, 0, 5), // "Board"
QT_MOC_LITERAL(1, 6, 17), // "completionChanged"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 5), // "value"
QT_MOC_LITERAL(4, 31, 15), // "overviewToggled"
QT_MOC_LITERAL(5, 47, 7), // "visible"
QT_MOC_LITERAL(6, 55, 23), // "retrievePiecesAvailable"
QT_MOC_LITERAL(7, 79, 9), // "available"
QT_MOC_LITERAL(8, 89, 11), // "showMessage"
QT_MOC_LITERAL(9, 101, 7), // "message"
QT_MOC_LITERAL(10, 109, 7), // "timeout"
QT_MOC_LITERAL(11, 117, 12), // "clearMessage"
QT_MOC_LITERAL(12, 130, 15), // "zoomInAvailable"
QT_MOC_LITERAL(13, 146, 16), // "zoomOutAvailable"
QT_MOC_LITERAL(14, 163, 11), // "zoomChanged"
QT_MOC_LITERAL(15, 175, 5), // "level"
QT_MOC_LITERAL(16, 181, 6), // "factor"
QT_MOC_LITERAL(17, 188, 8), // "finished"
QT_MOC_LITERAL(18, 197, 7), // "newGame"
QT_MOC_LITERAL(19, 205, 5), // "image"
QT_MOC_LITERAL(20, 211, 10), // "difficulty"
QT_MOC_LITERAL(21, 222, 8), // "openGame"
QT_MOC_LITERAL(22, 231, 2), // "id"
QT_MOC_LITERAL(23, 234, 8), // "saveGame"
QT_MOC_LITERAL(24, 243, 14), // "retrievePieces"
QT_MOC_LITERAL(25, 258, 6), // "zoomIn"
QT_MOC_LITERAL(26, 265, 7), // "zoomOut"
QT_MOC_LITERAL(27, 273, 7), // "zoomFit"
QT_MOC_LITERAL(28, 281, 4), // "zoom"
QT_MOC_LITERAL(29, 286, 14) // "toggleOverview"

    },
    "Board\0completionChanged\0\0value\0"
    "overviewToggled\0visible\0retrievePiecesAvailable\0"
    "available\0showMessage\0message\0timeout\0"
    "clearMessage\0zoomInAvailable\0"
    "zoomOutAvailable\0zoomChanged\0level\0"
    "factor\0finished\0newGame\0image\0difficulty\0"
    "openGame\0id\0saveGame\0retrievePieces\0"
    "zoomIn\0zoomOut\0zoomFit\0zoom\0toggleOverview"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Board[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  109,    2, 0x06 /* Public */,
       4,    1,  112,    2, 0x06 /* Public */,
       6,    1,  115,    2, 0x06 /* Public */,
       8,    2,  118,    2, 0x06 /* Public */,
       8,    1,  123,    2, 0x26 /* Public | MethodCloned */,
      11,    0,  126,    2, 0x06 /* Public */,
      12,    1,  127,    2, 0x06 /* Public */,
      13,    1,  130,    2, 0x06 /* Public */,
      14,    2,  133,    2, 0x06 /* Public */,
      17,    0,  138,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      18,    2,  139,    2, 0x0a /* Public */,
      21,    1,  144,    2, 0x0a /* Public */,
      23,    0,  147,    2, 0x0a /* Public */,
      24,    0,  148,    2, 0x0a /* Public */,
      25,    0,  149,    2, 0x0a /* Public */,
      26,    0,  150,    2, 0x0a /* Public */,
      27,    0,  151,    2, 0x0a /* Public */,
      28,    1,  152,    2, 0x0a /* Public */,
      29,    0,  155,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    9,   10,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Int, QMetaType::Float,   15,   16,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::Int,   19,   20,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void,

       0        // eod
};

void Board::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Board *_t = static_cast<Board *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->completionChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->overviewToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->retrievePiecesAvailable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->showMessage((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 4: _t->showMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->clearMessage(); break;
        case 6: _t->zoomInAvailable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->zoomOutAvailable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->zoomChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 9: _t->finished(); break;
        case 10: _t->newGame((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 11: _t->openGame((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->saveGame(); break;
        case 13: _t->retrievePieces(); break;
        case 14: _t->zoomIn(); break;
        case 15: _t->zoomOut(); break;
        case 16: _t->zoomFit(); break;
        case 17: _t->zoom((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->toggleOverview(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (Board::*_t)(int );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::completionChanged)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (Board::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::overviewToggled)) {
                *result = 1;
                return;
            }
        }
        {
            typedef void (Board::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::retrievePiecesAvailable)) {
                *result = 2;
                return;
            }
        }
        {
            typedef void (Board::*_t)(const QString & , int );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::showMessage)) {
                *result = 3;
                return;
            }
        }
        {
            typedef void (Board::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::clearMessage)) {
                *result = 5;
                return;
            }
        }
        {
            typedef void (Board::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::zoomInAvailable)) {
                *result = 6;
                return;
            }
        }
        {
            typedef void (Board::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::zoomOutAvailable)) {
                *result = 7;
                return;
            }
        }
        {
            typedef void (Board::*_t)(int , float );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::zoomChanged)) {
                *result = 8;
                return;
            }
        }
        {
            typedef void (Board::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::finished)) {
                *result = 9;
                return;
            }
        }
    }
}

const QMetaObject Board::staticMetaObject = {
    { &GLWidget::staticMetaObject, qt_meta_stringdata_Board.data,
      qt_meta_data_Board,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *Board::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Board::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_Board.stringdata0))
        return static_cast<void*>(const_cast< Board*>(this));
    return GLWidget::qt_metacast(_clname);
}

int Board::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GLWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void Board::completionChanged(int _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Board::overviewToggled(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Board::retrievePiecesAvailable(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Board::showMessage(const QString & _t1, int _t2)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 5
void Board::clearMessage()
{
    QMetaObject::activate(this, &staticMetaObject, 5, Q_NULLPTR);
}

// SIGNAL 6
void Board::zoomInAvailable(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void Board::zoomOutAvailable(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void Board::zoomChanged(int _t1, float _t2)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void Board::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 9, Q_NULLPTR);
}
QT_END_MOC_NAMESPACE
