/****************************************************************************
** Meta object code from reading C++ file 'new_game_tab.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/tetzle-2.1.1/src/new_game_tab.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'new_game_tab.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_NewGameTab_t {
    QByteArrayData data[19];
    char stringdata0[208];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_NewGameTab_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_NewGameTab_t qt_meta_stringdata_NewGameTab = {
    {
QT_MOC_LITERAL(0, 0, 10), // "NewGameTab"
QT_MOC_LITERAL(1, 11, 12), // "imageRenamed"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 5), // "image"
QT_MOC_LITERAL(4, 31, 4), // "name"
QT_MOC_LITERAL(5, 36, 7), // "newGame"
QT_MOC_LITERAL(6, 44, 10), // "difficulty"
QT_MOC_LITERAL(7, 55, 6), // "accept"
QT_MOC_LITERAL(8, 62, 15), // "addImageClicked"
QT_MOC_LITERAL(9, 78, 11), // "removeImage"
QT_MOC_LITERAL(10, 90, 19), // "editImageProperties"
QT_MOC_LITERAL(11, 110, 13), // "imageSelected"
QT_MOC_LITERAL(12, 124, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(13, 141, 4), // "item"
QT_MOC_LITERAL(14, 146, 17), // "pieceCountChanged"
QT_MOC_LITERAL(15, 164, 5), // "value"
QT_MOC_LITERAL(16, 170, 12), // "filterImages"
QT_MOC_LITERAL(17, 183, 6), // "filter"
QT_MOC_LITERAL(18, 190, 17) // "updateTagsStrings"

    },
    "NewGameTab\0imageRenamed\0\0image\0name\0"
    "newGame\0difficulty\0accept\0addImageClicked\0"
    "removeImage\0editImageProperties\0"
    "imageSelected\0QListWidgetItem*\0item\0"
    "pieceCountChanged\0value\0filterImages\0"
    "filter\0updateTagsStrings"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_NewGameTab[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   64,    2, 0x06 /* Public */,
       5,    2,   69,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,   74,    2, 0x08 /* Private */,
       8,    0,   75,    2, 0x08 /* Private */,
       9,    0,   76,    2, 0x08 /* Private */,
      10,    0,   77,    2, 0x08 /* Private */,
      11,    1,   78,    2, 0x08 /* Private */,
      14,    1,   81,    2, 0x08 /* Private */,
      16,    1,   84,    2, 0x08 /* Private */,
      18,    0,   87,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,    4,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    3,    6,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void, QMetaType::QStringList,   17,
    QMetaType::Void,

       0        // eod
};

void NewGameTab::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        NewGameTab *_t = static_cast<NewGameTab *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->imageRenamed((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 1: _t->newGame((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 2: _t->accept(); break;
        case 3: _t->addImageClicked(); break;
        case 4: _t->removeImage(); break;
        case 5: _t->editImageProperties(); break;
        case 6: _t->imageSelected((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 7: _t->pieceCountChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->filterImages((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 9: _t->updateTagsStrings(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (NewGameTab::*_t)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&NewGameTab::imageRenamed)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (NewGameTab::*_t)(const QString & , int );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&NewGameTab::newGame)) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject NewGameTab::staticMetaObject = {
    { &QWidget::staticMetaObject, qt_meta_stringdata_NewGameTab.data,
      qt_meta_data_NewGameTab,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *NewGameTab::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NewGameTab::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_NewGameTab.stringdata0))
        return static_cast<void*>(const_cast< NewGameTab*>(this));
    return QWidget::qt_metacast(_clname);
}

int NewGameTab::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void NewGameTab::imageRenamed(const QString & _t1, const QString & _t2)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void NewGameTab::newGame(const QString & _t1, int _t2)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_END_MOC_NAMESPACE
